
############ KANONICKA KORELACNI ANALYZA (CCA) - reseni ############

library(readxl) # balik pro funkci read_excel
library(CCA)    # balik pro CCA
library(CCP)    # balik pro testy signifikance kanonickych korelaci

setwd("c:/Users/koritakova/Documents/Vyuka/TELSON/2019-09_LS/TBL_CCA/04_priklad")
data1 <- read_excel("CCA_data.xlsx") # nacteni dat
names(data1) # nazvy jednotlivych promennych
# Jedna se o fiktivni data Factor.sta ze softwaru Statistica, ktera byla castecne modifikovana.
# Data sestavaji z 10 otazek tykajicich se spokojenosti s praci, konicky, domovem a obecnou spokojenosti a 2 charakteristik respondentu - pohlavi (0 pro muze a 1 pro zeny) a veku.
# Cilem je zjistit, jak spokojenost s praci ovlivnuje spokojenost v osobnim zivote (tzn. spokojenosti doma, spokojenosti s konicky a obecnou spokojenosti).


### 1) Overeni, zda se v datech vyskytuji chybejici hodnoty:

colSums(is.na(data1)) # pocet chybejicich hodnot v jednotlivych promennych

# u kterych subjektu jsou chybejici hodnoty:
data1[is.na(data1$WORK_1),] # tri 15-ti leti teenageri (kteri zrejme jeste nepracuji)

# odstraneni chybejicich hodnot:
data1 = data1[!is.na(data1$WORK_1),]

# overeni, ze uz tam zadne dalsi chybejici hodnoty nejsou:
colSums(is.na(data1))


### 2) Overeni vyskytu odlehlych hodnot a linearity:

# maticovy graf (pro overeni linearity a odlehlych hodnot):
pairs(~., panel= function(x,y,...) {
  points(x, y,...)
  abline(lm(y~x),col="grey")}, col="black", data=data1[,4:ncol(data1)]) # vykresleni grafu vcetne pridani primky linearni regrese
  # vidime 2 odlehle hodnoty (napr. u teckoveho grafu pro WORK_1 vs. HOME_1)

# odstraneni 1. odlehle hodnoty - subjektu s nejmensi pracovni spokojenosti:
data1[data1$WORK_1==min(data1$WORK_1),] # co je to za subjekt - nespokojeny teenager s brigadou i zivotem krome konicku
data1 <- data1[!(data1$WORK_1==min(data1$WORK_1)),] # odstraneni

# odstraneni 2. odlehle hodnoty - subjektu s nejmensi spokojenosti doma:
data1[data1$HOME_1==min(data1$HOME_1),] # co je to za subjekt - pracujici duchodkyne nespokojena s domacim zivotem (proto zrejme pracuje)
data1 <- data1[!(data1$HOME_1==min(data1$HOME_1)),] # odstraneni

# maticovy graf (pro overeni linearity a odlehlych hodnot) po odstraneni 2 odlehlych hodnot:
pairs(~., panel= function(x,y,...) {
  points(x, y,...)
  abline(lm(y~x),col="grey")}, col="black", data=data1[,4:ncol(data1)]) # vykresleni grafu vcetne pridani primky linearni regrese
  # dalsi odlehle hodnoty tam nejsou a zavislosti jsou linearni


### 3) Overeni normality:

P1 = data1[,4:6] # spokojenost s praci
#P1 = data1[,c("WORK_1","WORK_2","WORK_3")] # spokojenost s praci - 2. zpusob vyberu promennych
P2 = data1[,7:ncol(data1)] # spokojenost s osobnim zivotem
P = cbind(P1,P2) # vsechny promenne tykajici se spokojenosti

# histogramy (pro overeni normality):
hist(data1$WORK_1,prob=TRUE)
curve(dnorm(x, mean=mean(data1$WORK_1), sd=sd(data1$WORK_1)), col="darkblue", lwd=2, add=TRUE, yaxt="n")

# vykresleni vsech histogramu naraz:
for (i in 1:ncol(P)) {
  hist(as.numeric(P[[i]]),prob=TRUE,main=paste('Histogram of',names(P)[i]))
  curve(dnorm(x, mean=mean(P[[i]]), sd=sd(P[[i]])), col="darkblue", lwd=2, add=TRUE, yaxt="n")
}


### 4) Vypocet a vizualizace korelaci:

# korelace:
cor(P1) # vnitrni korelace promennych tykajicich se spokojenosti s praci
cor(P2) # vnitrni korelace promennych tykajicich se spokojenosti s osobnim zivotem
cor(P1,P2) # korelace promennych tykajicich se spokojenosti v praci s promennymi tykajicimi se spokojenosti v osobnim zivotem

# vizualizace korelacnich matic:
correl=matcor(P1,P2)
img.matcor(correl) # vizualizace korelaci mezi vsemi promennymi soucasne
img.matcor(correl,type=2) # vizualizace vnitrnich korelaci v jednotlivych skupinach promennych i korelaci mezi skupinami


### 5) Kanonicka korelacni analyza:

fit = cc(P1,P2) # vypocet CCA

p.asym(fit$cor, N=nrow(data1), p=ncol(P1), q=ncol(P2)) # asymptoticke testy korelaci 
       # Postupne do testu zarazuje vice a vice kanonickych korelaci, tzn. tabulku cteme od konce.  
       # 3. radek (test plneho modelu vs. model bez 3. paru kan. promennych): p-hodnota je 0.591, tedy nevyznamna -> tzn. modely jsou ekvivalentni, tzn. 3. par kan. promennych neni potreba.
       # 2. radek (test plneho modelu vs. model s odstranenim 2. a 3. paru kan. promennych): p-hodnota je 0.542, tzn. i ten 2. par kan. promennych muzeme vyradit.
       # 1. radek je signifikantni, tzn. vsechny 3 pary vyradit nemuzeme.
       # Tzn. pro vysvetleni vztahu mezi sadami puvodnich promennych staci jen 1. par kanonickych promennych (a mezi sadami existuje vztah).

fit # vysledkem 3 kanonicke korelace (neni mozne vytvorit vice promennych, protoze v P1 jsou jen 3 promenne)
    # 1. kanonicky korelacni koeficient je 0.88 (coz je vysoka korelace - ukazuje to, ze tam nejaky vztah je)
    # $xcoef - kanonicke vahy pro promenne P1
    # $ycoef - kanonicke vahy pro promenne P2
    # $scores$corr.X.xscores - kanonicke zateze pro promenne P1 (tzn. korelace kanonickych promennych vytvorenych z promennych P1 s puvodnimi promennymi P1)
    # $scores$corr.Y.yscores - kanonicke zateze pro promenne P2 (tzn. korelace kanonickych promennych vytvorenych z promennych P2 s puvodnimi promennymi P2)
    # Interpretace 1. paru kanonickych promennych:
          # S prvni kanonickou promennou silne koreluji vsechny promenne tykajici se spokojenosti v praci.
          # S druhou kanonickou promennou silne koreluji promenne tykajici se spokojenosti s konicky a obecnou spokojenosti.
          # Protoze jsou vsechny tyto promenne silne (vnitrne) korelovany, coz vidime z cor(P1) a cor(P2), jsou vsechny kanonicke vahy velmi nizke.
          # Pro interpretaci je tedy v tomto pripade vhodne vyuzit predevsim kanonicke zateze.
          # Zaver: Cim nizsi spokojenost v praci, tim nizsi obecna spokojenost i spokojenost s konicky. Spokojenost doma neni spokojenosti s praci prilis ovlivnena.


write.table(fit$cor,"CCA_reseni.txt") # ulozeni do textoveho souboru (pro usnadneni interpretace kan. promennych)
write.table(fit$xcoef,"CCA_reseni.txt",append=TRUE)
write.table(fit$ycoef,"CCA_reseni.txt",append=TRUE)
write.table(fit$scores$corr.X.xscores,"CCA_reseni.txt",append=TRUE)
write.table(fit$scores$corr.Y.yscores,"CCA_reseni.txt",append=TRUE)


# vizualizace subjektu v novem prostoru 1. paru kanonickych promennych:
plot(fit$scores$xscores[,1],fit$scores$yscores[,1])
abline(lm(fit$scores$yscores[,1]~fit$scores$xscores[,1]),col="grey")
    # Zhodnoceni: 
    # - mezi promennymi 1. paru je skutecne linearni vztah - je splnen jeden z predpokladu CCA
    # - v datech se nevyskytuji odlehle hodnoty - je splnen dalsi z predpokladu CCA
    # - v datech se nevyskytuji skupiny subjektu (napr. jedna skupina, ktera by byla hodne nespokojena, a druha hodne spokojena)
    #   (kdyby se ukazalo, ze tam skupiny jsou, bylo by vhodne zjistit, co je to za skupiny)
    # Tzn. i kdyz jsme overovali predpoklady na zacatku pred CCA, je vhodne linearitu a odlehle hodnoty overit i ve vysledku. A navic se podivat, zda jsou v datech skupiny.


# vizualizace prvnich dvou novych "dimenzi"
plt.cc(fit, type="v", var.label=T) 
    # Zhodnoceni:
    # - protoze byla vyznamna jen 1. kan. korelace, v tomto pripade ma smysl se divat jen na 1. dimenzi (tzn. je vhodne si 2. dimenzi odmyslet)
    # - vidime, ze 3 promenne tykajici se spokojenosti s praci jsou blizko promennych tykajicich se spokojenosti s konicky a obecnou spokojenosti
    #   (takze tyto promenne spolu nejvice souvisi); zatimco promenne tykajici se spokojenosti doma s ostatnimi prilis nesouvisi
