
############ KANONICKA KORELACNI ANALYZA (CCA) - zadani ############

library(readxl) # balik pro funkci read_excel
library(CCA)    # balik pro CCA
library(CCP)    # balik pro testy signifikance kanonickych korelaci

setwd("c:/Users/koritakova/Documents/Vyuka/TELSON/2019-09_LS/TBL_CCA/04_priklad") # ukazka nastaveni cesty
data1 <- read_excel("CCA_data.xlsx") # nacteni dat
names(data1) # nazvy jednotlivych promennych
# Jedna se o fiktivni data Factor.sta ze softwaru Statistica, ktera byla castecne modifikovana.
# Data sestavaji z 10 otazek tykajicich se spokojenosti s praci, konicky, domovem a obecnou spokojenosti a 2 charakteristik respondentu - pohlavi (0 pro muze a 1 pro zeny) a veku.


# UKOL: Zjistete, jak spokojenost s praci ovlivnuje spokojenost v osobnim zivote (tzn. spokojenosti doma, spokojenosti s konicky a obecnou spokojenosti).



############ CO VAM MUZE BYT NAPOMOCNE ############ 

### Manipulace s daty:

# Vyber promennych (ukazka 2 moznych zpusobu):
P1 = data1[,4:6] # 1. zpusob vyberu promennych
P1 = data1[,c("WORK_1","WORK_2","WORK_3")] # 2. zpusob vyberu promennych

# Odstraneni chybejicich hodnot:
data1 = data1[!is.na(data1$ID),] # odstraneni chybejicich hodnot (pokud by byly chybejici hodnoty v promenne ID)


### Overovani predpokladu:

# sumarizace poctu chybejicich hodnot v jednotlivych promennych:
colSums(is.na(data1))

# maticovy graf:
pairs(~., panel= function(x,y,...) {
  points(x, y,...)
  abline(lm(y~x),col="grey")}, col="black", data=data1) # vykresleni grafu vcetne pridani primky linearni regrese

# histogram:
hist(data1$WORK_1,prob=TRUE) # vykresleni histogramu
curve(dnorm(x, mean=mean(data1$WORK_1), sd=sd(data1$WORK_1)), col="darkblue", lwd=2, add=TRUE, yaxt="n") # pridani krivky normalniho rozdeleni


### Vypocet a vizualizace korelaci:

# korelace:
cor(P1) # vnitrni korelace promennych uvnitr jedne skupiny
cor(P1,P2) # korelace promennych mezi dvema skupinami promennych

# vizualizace korelacnich matic:
correl=matcor(P1,P2)
img.matcor(correl) # vizualizace korelaci mezi vsemi promennymi soucasne
img.matcor(correl,type=2) # vizualizace vnitrnich korelaci v jednotlivych skupinach promennych i korelaci mezi skupinami


### Kanonicka korelacni analyza:

fit = cc(P1,P2) # vypocet CCA
p.asym(fit$cor, N=nrow(data1), p=ncol(P1), q=ncol(P2)) # asymptoticke testy korelaci 
fit # vysledek CCA (pouzijte manual https://cran.r-project.org/web/packages/CCA/CCA.pdf a zjistete, co znamenaji jednotlive vystupy)


### Vizualizace vystupu z CCA (neni nutne pouzit, okomentujeme si pak spolecne):

# vizualizace subjektu v novem prostoru 1. paru kanonickych promennych:
plot(fit$scores$xscores[,1],fit$scores$yscores[,1])
abline(lm(fit$scores$yscores[,1]~fit$scores$xscores[,1]),col="grey")

# vizualizace prvnich dvou novych "dimenzi"
plt.cc(fit, type="v", var.label=T) 


### Dalsi pomocne funkce:

write.table(cor(P1),"CCA_reseni.txt") # ulozeni do textoveho souboru (pro usnadneni interpretace)
write.table(cor(P1,P2),"CCA_reseni.txt",append=TRUE) # pridani dalsiho vystupu do textoveho souboru (pro usnadneni interpretace)
